/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.agent;

import java.util.Random;

import org.xmloperator.reactive.CrashingReport;
import org.xmloperator.reactive.PrintStream;
import org.xmloperator.reactive.ReactiveFactory;
import org.xmloperator.reactive.ReactiveMachine;
import org.xmloperator.reactive.impl.ReactiveCommunityImpl;
import org.xmloperator.reactive.impl.ReactiveEntityImpl;

/**
 * ReactiveFactory implementation.
 */
public final class ReactiveFactoryImpl implements ReactiveFactory {
	private final Random random = new Random(); 
	
	/**
	 * Unique instance of ReactiveFactoryImpl.
	 */
	public static final ReactiveFactoryImpl REACTIVE_FACTORY =
		new ReactiveFactoryImpl();
	
	/**
	 * Activates the trace system. Traces are activated at a static level, i.e.
	 * for all the instances of these classes.
	 * 
	 * @param trace
	 *            a PrintStream for receiving the traces.
	 * @param traceSpec
	 *            a char set for selecting concerned classes. Lower case is for
	 *            standard trace. Upper case is for full trace. C/c :
	 *            ReactiveCommunityImpl, R/r : ReactiveEntityImpl, T/t :
	 *            AgentThread, m : ReactiveMachineImpl, a : AgentImpl, o :
	 *            ActivityObserverImpl
	 */
	public static void setTrace(PrintStream trace, String traceSpec) {
		{
			boolean _C = traceSpec.indexOf('C') >= 0;
			boolean _c = traceSpec.indexOf('c') >= 0;
			ReactiveCommunityImpl.setTrace((_C  | _c) ? trace : null, _C);
		}
		{
			boolean _R = traceSpec.indexOf('R') >= 0;
			boolean _r = traceSpec.indexOf('r') >= 0;
			ReactiveEntityImpl.setTrace((_R  | _r) ? trace : null, _R);
		}
		{
			boolean _T = traceSpec.indexOf('T') >= 0;
			boolean _t = traceSpec.indexOf('t') >= 0;
			AgentThread.setTrace((_T  | _t) ? trace : null, _T);
		}
		{
			boolean _m = traceSpec.indexOf('m') >= 0;
			boolean _a = traceSpec.indexOf('a') >= 0;
			ReactiveMachineImpl.setTrace(trace, _m, _a);

			boolean _o = traceSpec.indexOf('o') >= 0;
			ActivityObserverImpl.setTrace(_o ? trace : null);
		}
	}
	
	private ReactiveFactoryImpl() {
	}
	
	/////////////////////////////////
	// ReactiveFactory implementation
	/////////////////////////////////
	
	public long newKey() {
		while (true) {
			long key = this.random.nextLong() + new Random().nextLong();
			if (key != 0)
				return key;
		}
	}

	public ReactiveMachine newReactiveMachine(CrashingReport crashingReport,
			long machineDoneKey) {
		return new ReactiveMachineImpl(crashingReport, machineDoneKey);
	}
	
	public ReactiveMachine getReactiveMachine() {
		return ReactiveMachineImpl.getCurrentlyActiveMachine();
	}
}
