/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.market.impl;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;

import org.xmloperator.reactive.demo.market.Anticipation;
import org.xmloperator.reactive.demo.market.IncorrectAccountHolderException;
import org.xmloperator.reactive.demo.market.ProjectedAccount;
import org.xmloperator.reactive.demo.market.Bank;

/**
 * ProjectedAccount implementation.
 */
final class ProjectedAccountImpl extends AccountImpl
		implements ProjectedAccount {
	private List<Anticipation> anticipationList = new ArrayList<Anticipation>();
	private BigInteger negativeAnticipationSum = BigInteger.ZERO;
	private BigInteger positiveAnticipationSum = BigInteger.ZERO;
	
	ProjectedAccountImpl(Bank bank, long holderKey) {
		super(bank, holderKey);
	}

	//////////////////////////////////
	// ProjectedAccount implementation
	//////////////////////////////////
	
	public final BigInteger getSoldWithNegativeAnticipations(long holderKey) {
		if (holderKey != this.holderKey)
			throw new IncorrectAccountHolderException();
		return this.resourceUnitCount.add(this.negativeAnticipationSum);
	}

	public final BigInteger getSoldWithPositiveAnticipations(long holderKey) {
		if (holderKey != this.holderKey)
			throw new IncorrectAccountHolderException();
		return this.resourceUnitCount.add(this.positiveAnticipationSum);
	}

	public final void addAnticipation(long holderKey, Anticipation anticipation) {
		if (anticipation == null)
			throw new IllegalArgumentException();
		if (holderKey != this.holderKey)
			throw new IncorrectAccountHolderException();
		if (trace != null)
			trace.println("P" + id + " add(" + anticipation +")");
		synchronized (this.anticipationList) {
			this.anticipationList.add(anticipation);
			BigInteger anticipationUnitCount = anticipation
					.getResourceUnitCount();
			if (anticipationUnitCount.compareTo(BigInteger.ZERO) < 0)
				this.negativeAnticipationSum = this.negativeAnticipationSum
						.add(anticipationUnitCount);
			else
				this.positiveAnticipationSum = this.positiveAnticipationSum
						.add(anticipationUnitCount);
		}
	}

	public final void removeAnticipation(long holderKey, Anticipation anticipation) {
		if (anticipation == null)
			throw new IllegalArgumentException();
		if (holderKey != this.holderKey)
			throw new IncorrectAccountHolderException();
		if (trace != null)
			trace.println("P" + id + " remove(" + anticipation +")");
		synchronized (this.anticipationList) {
			if (!this.anticipationList.remove(anticipation))
				throw new IllegalStateException();
			BigInteger anticipationUnitCount = anticipation
					.getResourceUnitCount();
			if (anticipationUnitCount.compareTo(BigInteger.ZERO) < 0)
				this.negativeAnticipationSum = this.negativeAnticipationSum
						.subtract(anticipationUnitCount);
			else
				this.positiveAnticipationSum = this.positiveAnticipationSum
						.subtract(anticipationUnitCount);
		}
	}

	public final boolean hasAnticipations() {
		return !this.anticipationList.isEmpty();
	}
}
