/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.market.impl;

import org.xmloperator.reactive.demo.market.Bank;
import org.xmloperator.reactive.demo.market.Resource;

/**
 * Resource implementation.
 */
public class ResourceImpl implements Resource {
	private final String resourceName;
	private final Bank referenceBank;
	private double unitValue;
	private double weight;
	
	public ResourceImpl(String resourceName, long initiatorKey,
			double unitValue, double weight) {
		if (resourceName == null)
			throw new IllegalArgumentException();
		this.resourceName = resourceName;
		this.referenceBank = new BankImpl(this, initiatorKey);
		this.setUnitValue(unitValue);
		this.setWeight(weight);
	}
	
	final void setUnitValue(double unitValue) {
		if (unitValue <= 0.0)
			throw new IllegalArgumentException();
		this.unitValue = unitValue;
	}
	
	final void setWeight(double weight) {
		if (weight <= 0.0)
			throw new IllegalArgumentException();
		this.weight = weight;
	}
	
	//////////////////////////
	// Resource implementation
	//////////////////////////

	public final String getResourceName() {
		return this.resourceName;
	}

	public final Bank getReferenceBank() {
		return this.referenceBank;
	}
	
	public final double getUnitValue() {
		return this.unitValue;
	}
	
	public double getWeight() {
		return this.weight;
	}
	
	////////////////////////
	// Object implementation
	////////////////////////
	
	public String toString() {
		return this.resourceName;
	}
}
