/*
 * Copyright (c) 2008, 2011 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.demo.market.impl;

import java.math.BigInteger;

import org.xmloperator.reactive.demo.market.Account;
import org.xmloperator.reactive.demo.market.InsufficientSoldException;

import org.xmloperator.reactive.exception.ReactiveException;

/**
 * Operation that consists in a transfer.
 */
final class Transfer extends OperationImpl {
	private final long holderKey;
	private final Account sourceAccount;
	private final Account destinationAccount;
	private final BigInteger unitCount;
	
	Transfer(long holderKey, Account sourceAccount, Account destinationAccount,
			BigInteger unitCount) {
		if (sourceAccount == null || destinationAccount == null
				|| unitCount.compareTo(BigInteger.ZERO) < 0)
			throw new IllegalArgumentException();
		this.holderKey = holderKey;
		this.sourceAccount = sourceAccount;
		this.destinationAccount = destinationAccount;
		this.unitCount = unitCount;
	}

	///////////////////////////
	// Operation implementation
	///////////////////////////

	public void execute() throws ReactiveException {
		super.execute();
		if (!this.sourceAccount.transfer(this.holderKey,
				this.destinationAccount, this.unitCount))
			throw new InsufficientSoldException(this.sourceAccount);
	}
}
