/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.element;

import org.xmloperator.reactive.element.exception.ChildrenConstraintException;
import org.xmloperator.reactive.element.exception.IllegalElementChangeException;

/**
 * Document read-write element.
 * There is no deterministic setting of a data Element.
 */
public interface RWElement {

	/**
	 * Returns the associated Element.
	 * @return the associated Element.
	 */	
	public Element getElement();

	/**
	 * Insert an Element before this one. This must have a parent.
	 * @param element an Element to insert. It must have no parent and not to be
	 *   a document Element.
	 * @throws IllegalDocumentChangeException if this Element has no parent.
	 * @throws IllegalArgumentException if the given Element has a parent or
	 *   is the root Element of the hierarchy that contains this Element.
	 * @throws ChildrenConstraintException if this operation doesn't respect
	 *   a ChildrenConstraint on the parent of the given Element.
	 */
	public void insertBefore(Element element) throws
			IllegalElementChangeException,
			IllegalArgumentException,
			ChildrenConstraintException;
	
	/**
	 * Removes this Element.
	 * Do nothing if this Element has no parent.
	 */
	public void remove() throws IllegalElementChangeException;
}
