/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.element.constraint;

import org.xmloperator.reactive.element.exception.ChildrenConstraintException;

import org.xmloperator.reactive.element.ChildrenConstraint;
import org.xmloperator.reactive.element.Element;
import org.xmloperator.reactive.element.ParentElement;

/**
 * This ChildrenConstraint says that there must not exist two children
 *   with the same name.
 */
public final class DifferentNamesChildrenConstraint
		implements ChildrenConstraint {
	
	public static final DifferentNamesChildrenConstraint CHILDREN_CONSTRAINT =
		new DifferentNamesChildrenConstraint();
	
	private DifferentNamesChildrenConstraint() {
	}
	
	/////////////////////
	// ChildrenConstraint
	/////////////////////
	
	public void verifyConstraint(
			ParentElement parentElement,
			Element addedOrRemovedChildElement, boolean isChildAdded)
			throws ChildrenConstraintException {
		if (isChildAdded) {
			String namespaceURI =
					addedOrRemovedChildElement.getNamespaceURI();
			String localName = addedOrRemovedChildElement.getLocalName();
			Element child = parentElement.getFirstChild();
			while (child != null) {
				if (child.getNamespaceURI() == namespaceURI
						&& child.getLocalName().equals(localName)) {
//					System.err.println(
//						"AddElement(\""
//						+ namespaceURI
//						+ "\",\""
//						+ localName
//						+ "\")");
					throw new ChildrenConstraintException(this);
				}
				child = child.getNextSibling();
			}
		}
	}
	
	////////////////////////
	// Object implementation
	////////////////////////

	public String toString() {
		return "D";
	}
}
