/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.element.impl;


import org.xmloperator.reactive.element.Element;
import org.xmloperator.reactive.element.DataElement;
import org.xmloperator.reactive.element.StringDataElement;
import org.xmloperator.reactive.element.exception.BadCharInNameException;

/**
 * ObjectElement implementation.
 */
public final class StringDataElementImpl extends ElementImpl
		implements StringDataElement {

	final String data;

	public StringDataElementImpl(String namespaceURI, String tagName,
			String data) throws BadCharInNameException {
		super(namespaceURI, tagName);
		this.data = data;
	}

	///////////////////////////////////
	// StringDataElement implementation
	///////////////////////////////////
	
	public String getData() {
		return this.data;
	}

	/////////////////////////////
	// DataElement implementation
	/////////////////////////////
	
	public String getDataType() {
		return DataElement.TYPE_STRING;
	}
	
	public String stringDataValue() {
		return this.data;
	}
	
	/////////////////////////
	// Element implementation
	/////////////////////////
	
	public boolean isDataElement() {
		return true;
	}

	public boolean equals(Element element) {
		if (element == this)
			return true;
		if (!(element instanceof StringDataElementImpl)
				|| !super.equals(element))
			return false;
		StringDataElementImpl otherElement = (StringDataElementImpl)element;
		return (this.data == null && otherElement.data == null)
			|| (this.data != null && this.data.equals(otherElement.data));
	}

	////////////////////////
	// Object implementation
	////////////////////////
	
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append(super.toString());
		sb.append('(');
		if (this.data != null) {
			sb.append('"');
			sb.append(this.data);
			sb.append('"');
		}
		sb.append(')');
		return sb.toString();
	}
}
